@@ -1,6 +1,6 @@
 /* Target-dependent code for the Toshiba MeP for GDB, the GNU debugger.
 
-   Copyright (C) 2001-2021 Free Software Foundation, Inc.
+   Copyright (C) 2001-2026 Free Software Foundation, Inc.
 
    Contributed by Red Hat, Inc.
 
@@ -19,13 +19,13 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
+#include "extract-store-integer.h"
 #include "frame.h"
 #include "frame-unwind.h"
 #include "frame-base.h"
 #include "symtab.h"
 #include "gdbtypes.h"
-#include "gdbcmd.h"
+#include "cli/cli-cmds.h"
 #include "gdbcore.h"
 #include "value.h"
 #include "inferior.h"
@@ -37,7 +37,6 @@
 #include "regcache.h"
 #include "remote.h"
 #include "sim-regno.h"
-#include "disasm.h"
 #include "trad-frame.h"
 #include "reggroups.h"
 #include "elf-bfd.h"
@@ -45,9 +44,13 @@
 #include "prologue-value.h"
 #include "cgen/bitset.h"
 #include "infcall.h"
+#include "gdbarch.h"
 
 /* Get the user's customized MeP coprocessor register names from
-   libopcodes.  */
+   libopcodes.  Make cgen names unique to prevent ODR conflicts with other
+   targets.  */
+#define GDB_CGEN_REMAP_PREFIX mep
+#include "cgen-remap.h"
 #include "opcodes/mep-desc.h"
 #include "opcodes/mep-opc.h"
 
@@ -57,7 +60,7 @@
 /* A quick recap for GDB hackers not familiar with the whole Toshiba
    Media Processor story:
 
-   The MeP media engine is a configureable processor: users can design
+   The MeP media engine is a configurable processor: users can design
    their own coprocessors, implement custom instructions, adjust cache
    sizes, select optional standard facilities like add-and-saturate
    instructions, and so on.  Then, they can build custom versions of
@@ -116,7 +119,7 @@
      options are present on the current processor.  */
 
 
-struct gdbarch_tdep
+struct mep_gdbarch_tdep : gdbarch_tdep_base
 {
   /* A CGEN cpu descriptor for this BFD architecture and machine.
 
@@ -124,7 +127,7 @@ struct gdbarch_tdep
      MeP libopcodes machinery actually puts off module-specific
      customization until the last minute.  So this contains
      information about all supported me_modules.  */
-  CGEN_CPU_DESC cpu_desc;
+  CGEN_CPU_DESC cpu_desc = nullptr;
 
   /* The me_module index from the ELF file we used to select this
      architecture, or CONFIG_NONE if there was none.
@@ -140,7 +143,7 @@ struct gdbarch_tdep
      create a separate instance of the gdbarch structure for each
      me_module value mep_gdbarch_init sees, and store the me_module
      value from the ELF file here.  */
-  CONFIG_ATTR me_module;
+  CONFIG_ATTR me_module {};
 };
 
 
@@ -259,7 +262,9 @@ me_module_register_set (CONFIG_ATTR me_module,
        mask contains any of the me_module's coprocessor ISAs,
        specifically excluding the generic coprocessor register sets.  */
 
-  CGEN_CPU_DESC desc = gdbarch_tdep (target_gdbarch ())->cpu_desc;
+  mep_gdbarch_tdep *tdep
+    = gdbarch_tdep<mep_gdbarch_tdep> (current_inferior ()->arch ());
+  CGEN_CPU_DESC desc = tdep->cpu_desc;
   const CGEN_HW_ENTRY *hw;
 
   if (me_module == CONFIG_NONE)
@@ -327,7 +332,7 @@ register_name_from_keyword (CGEN_KEYWORD *keyword_table, int regnum)
     return "";
 }
 
-  
+
 /* Masks for option bits in the OPT special-purpose register.  */
 enum {
   MEP_OPT_DIV = 1 << 25,        /* 32-bit divide instruction option */
@@ -432,7 +437,7 @@ me_module_name (CONFIG_ATTR me_module)
 
 
 /* The MeP spec defines the following registers:
-   16 general purpose registers (r0-r15) 
+   16 general purpose registers (r0-r15)
    32 control/special registers (csr0-csr31)
    32 coprocessor general-purpose registers (c0 -- c31)
    64 coprocessor control registers (ccr0 -- ccr63)
@@ -847,12 +852,16 @@ current_me_module (void)
   if (target_has_registers ())
     {
       ULONGEST regval;
-      regcache_cooked_read_unsigned (get_current_regcache (),
+      regcache_cooked_read_unsigned (get_thread_regcache (inferior_thread ()),
 				     MEP_MODULE_REGNUM, &regval);
       return (CONFIG_ATTR) regval;
     }
   else
-    return gdbarch_tdep (target_gdbarch ())->me_module;
+    {
+      mep_gdbarch_tdep *tdep
+	= gdbarch_tdep<mep_gdbarch_tdep> (current_inferior ()->arch ());
+      return tdep->me_module;
+    }
 }
 
 
@@ -870,7 +879,7 @@ current_options (void)
   if (target_has_registers ())
     {
       ULONGEST regval;
-      regcache_cooked_read_unsigned (get_current_regcache (),
+      regcache_cooked_read_unsigned (get_thread_regcache (inferior_thread ()),
 				     MEP_OPT_REGNUM, &regval);
       return regval;
     }
@@ -937,7 +946,7 @@ mep_register_name (struct gdbarch *gdbarch, int regnr)
 
   /* Special-purpose registers.  */
   static const char *csr_names[] = {
-    "pc",   "lp",   "sar",  "",     /* 0  csr3: reserved */ 
+    "pc",   "lp",   "sar",  "",     /* 0  csr3: reserved */
     "rpb",  "rpe",  "rpc",  "hi",   /* 4 */
     "lo",   "",     "",     "",     /* 8  csr9-csr11: reserved */
     "mb0",  "me0",  "mb1",  "me1",  /* 12 */
@@ -1010,19 +1019,19 @@ mep_register_name (struct gdbarch *gdbarch, int regnr)
      would affect the output of 'info all-registers', which would
      disturb the test suites.  So we leave it invisible.  */
   else
-    return NULL;
+    return "";
 }
 
 
 /* Custom register groups for the MeP.  */
-static struct reggroup *mep_csr_reggroup; /* control/special */
-static struct reggroup *mep_cr_reggroup;  /* coprocessor general-purpose */
-static struct reggroup *mep_ccr_reggroup; /* coprocessor control */
+static const reggroup *mep_csr_reggroup; /* control/special */
+static const reggroup *mep_cr_reggroup;  /* coprocessor general-purpose */
+static const reggroup *mep_ccr_reggroup; /* coprocessor control */
 
 
 static int
 mep_register_reggroup_p (struct gdbarch *gdbarch, int regnum,
-			 struct reggroup *group)
+			 const struct reggroup *group)
 {
   /* Filter reserved or unused register numbers.  */
   {
@@ -1125,8 +1134,8 @@ mep_pseudo_cr32_read (struct gdbarch *gdbarch,
   int rawnum = mep_pseudo_to_raw[cookednum];
   gdb_byte buf64[8];
 
-  gdb_assert (TYPE_LENGTH (register_type (gdbarch, rawnum)) == sizeof (buf64));
-  gdb_assert (TYPE_LENGTH (register_type (gdbarch, cookednum)) == 4);
+  gdb_assert (register_type (gdbarch, rawnum)->length () == sizeof (buf64));
+  gdb_assert (register_type (gdbarch, cookednum)->length () == 4);
   status = regcache->raw_read (rawnum, buf64);
   if (status == REG_VALID)
     {
@@ -1189,7 +1198,7 @@ mep_pseudo_csr_write (struct gdbarch *gdbarch,
       ULONGEST old_bits;
       ULONGEST new_bits;
       ULONGEST mixed_bits;
-	  
+
       regcache_raw_read_unsigned (regcache, r->raw, &old_bits);
       new_bits = extract_unsigned_integer (buf, size, byte_order);
       mixed_bits = ((r->writeable_bits & new_bits)
@@ -1197,7 +1206,7 @@ mep_pseudo_csr_write (struct gdbarch *gdbarch,
       regcache_raw_write_unsigned (regcache, r->raw, mixed_bits);
     }
 }
-		      
+
 
 static void
 mep_pseudo_cr32_write (struct gdbarch *gdbarch,
@@ -1210,9 +1219,9 @@ mep_pseudo_cr32_write (struct gdbarch *gdbarch,
      the pseudoregister.  */
   int rawnum = mep_pseudo_to_raw[cookednum];
   gdb_byte buf64[8];
-  
-  gdb_assert (TYPE_LENGTH (register_type (gdbarch, rawnum)) == sizeof (buf64));
-  gdb_assert (TYPE_LENGTH (register_type (gdbarch, cookednum)) == 4);
+
+  gdb_assert (register_type (gdbarch, rawnum)->length () == sizeof (buf64));
+  gdb_assert (register_type (gdbarch, cookednum)->length () == 4);
   /* Slow, but legible.  */
   store_unsigned_integer (buf64, 8, byte_order,
 			  extract_unsigned_integer (buf, 4, byte_order));
@@ -1377,7 +1386,7 @@ mep_pc_in_vliw_section (CORE_ADDR pc)
    So, the *INSN values for the instruction sequence above would be
    the following, in either endianness:
 
-       0xd1561234       movu $1,0x123456     
+       0xd1561234       movu $1,0x123456
        0xc1285678 	sb $1,22136($2)
        0xf1011098 	clip $1,0x13
        0x70020000      	ret
@@ -1399,7 +1408,7 @@ mep_pc_in_vliw_section (CORE_ADDR pc)
    significant to prologue analysis --- for the time being,
    anyway.  */
 
-static CORE_ADDR 
+static CORE_ADDR
 mep_get_insn (struct gdbarch *gdbarch, CORE_ADDR pc, unsigned long *insn)
 {
   enum bfd_endian byte_order = gdbarch_byte_order (gdbarch);
@@ -1470,7 +1479,7 @@ mep_get_insn (struct gdbarch *gdbarch, CORE_ADDR pc, unsigned long *insn)
       else
 	gdb_assert_not_reached ("unexpected vliw mode");
     }
-  
+
   /* Otherwise, the top two bits of the major opcode are (again) what
      we need to check.  */
   else if ((*insn & 0xc0000000) == 0xc0000000)
@@ -1727,7 +1736,7 @@ mep_analyze_prologue (struct gdbarch *gdbarch,
 	     accuracy, it would be better to just quit now.  */
 	  if (stack.store_would_trash (reg[rm]))
 	    break;
-	  
+
 	  if (is_arg_spill (gdbarch, reg[rn], reg[rm], &stack))
 	    after_last_frame_setup_insn = next_pc;
 
@@ -1803,11 +1812,11 @@ mep_analyze_prologue (struct gdbarch *gdbarch,
 	     to this branch target and also stop the prologue scan.
 	     The instructions at and beyond the branch target should
 	     no longer be associated with the prologue.
-	     
+
 	     Note that we only consider forward branches here.  We
 	     presume that a forward branch is being used to skip over
 	     a loop body.
-	     
+
 	     A backwards branch is covered by the default case below.
 	     If we were to encounter a backwards branch, that would
 	     most likely mean that we've scanned through a loop body.
@@ -1833,7 +1842,7 @@ mep_analyze_prologue (struct gdbarch *gdbarch,
 
 	 - If the instruction just changed the FP back to its original
 	   value, then that's probably a restore instruction.  The
-	   prologue should definitely end before that.  
+	   prologue should definitely end before that.
 
 	 - If the instruction increased the value of the SP (that is,
 	   shrunk the frame), then it's probably part of a frame
@@ -1904,15 +1913,14 @@ typedef BP_MANIPULATION (mep_break_insn) mep_breakpoint;
 
 
 static struct mep_prologue *
-mep_analyze_frame_prologue (struct frame_info *this_frame,
+mep_analyze_frame_prologue (const frame_info_ptr &this_frame,
 			    void **this_prologue_cache)
 {
   if (! *this_prologue_cache)
     {
       CORE_ADDR func_start, stop_addr;
 
-      *this_prologue_cache 
-	= FRAME_OBSTACK_ZALLOC (struct mep_prologue);
+      *this_prologue_cache = frame_obstack_zalloc<mep_prologue> ();
 
       func_start = get_frame_func (this_frame);
       stop_addr = get_frame_pc (this_frame);
@@ -1934,7 +1942,7 @@ mep_analyze_frame_prologue (struct frame_info *this_frame,
 /* Given the next frame and a prologue cache, return this frame's
    base.  */
 static CORE_ADDR
-mep_frame_base (struct frame_info *this_frame,
+mep_frame_base (const frame_info_ptr &this_frame,
 		void **this_prologue_cache)
 {
   struct mep_prologue *p
@@ -1962,7 +1970,7 @@ mep_frame_base (struct frame_info *this_frame,
 
 
 static void
-mep_frame_this_id (struct frame_info *this_frame,
+mep_frame_this_id (const frame_info_ptr &this_frame,
 		   void **this_prologue_cache,
 		   struct frame_id *this_id)
 {
@@ -1972,7 +1980,7 @@ mep_frame_this_id (struct frame_info *this_frame,
 
 
 static struct value *
-mep_frame_prev_register (struct frame_info *this_frame,
+mep_frame_prev_register (const frame_info_ptr &this_frame,
 			 void **this_prologue_cache, int regnum)
 {
   struct mep_prologue *p
@@ -2052,15 +2060,16 @@ mep_frame_prev_register (struct frame_info *this_frame,
 }
 
 
-static const struct frame_unwind mep_frame_unwind = {
+static const struct frame_unwind_legacy mep_frame_unwind (
   "mep prologue",
   NORMAL_FRAME,
+  FRAME_UNWIND_ARCH,
   default_frame_unwind_stop_reason,
   mep_frame_this_id,
   mep_frame_prev_register,
   NULL,
   default_frame_sniffer
-};
+);
 
 
 /* Return values.  */
@@ -2069,7 +2078,7 @@ static const struct frame_unwind mep_frame_unwind = {
 static int
 mep_use_struct_convention (struct type *type)
 {
-  return (TYPE_LENGTH (type) > MEP_GPR_SIZE);
+  return (type->length () > MEP_GPR_SIZE);
 }
 
 
@@ -2088,15 +2097,15 @@ mep_extract_return_value (struct gdbarch *arch,
 
   /* Return values > MEP_GPR_SIZE bytes are returned in memory,
      pointed to by R0.  */
-  gdb_assert (TYPE_LENGTH (type) <= MEP_GPR_SIZE);
+  gdb_assert (type->length () <= MEP_GPR_SIZE);
 
   if (byte_order == BFD_ENDIAN_BIG)
-    offset = MEP_GPR_SIZE - TYPE_LENGTH (type);
+    offset = MEP_GPR_SIZE - type->length ();
   else
     offset = 0;
 
   /* Return values that do fit in a single register are returned in R0.  */
-  regcache->cooked_read_part (MEP_R0_REGNUM, offset, TYPE_LENGTH (type),
+  regcache->cooked_read_part (MEP_R0_REGNUM, offset, type->length (),
 			      valbuf);
 }
 
@@ -2110,7 +2119,7 @@ mep_store_return_value (struct gdbarch *arch,
   int byte_order = gdbarch_byte_order (arch);
 
   /* Values that fit in a single register go in R0.  */
-  if (TYPE_LENGTH (type) <= MEP_GPR_SIZE)
+  if (type->length () <= MEP_GPR_SIZE)
     {
       /* Values that don't occupy a full register appear at the least
 	 significant end of the value.  This is the offset to where the
@@ -2118,11 +2127,11 @@ mep_store_return_value (struct gdbarch *arch,
       int offset;
 
       if (byte_order == BFD_ENDIAN_BIG)
-	offset = MEP_GPR_SIZE - TYPE_LENGTH (type);
+	offset = MEP_GPR_SIZE - type->length ();
       else
 	offset = 0;
 
-      regcache->cooked_write_part (MEP_R0_REGNUM, offset, TYPE_LENGTH (type),
+      regcache->cooked_write_part (MEP_R0_REGNUM, offset, type->length (),
 				   valbuf);
     }
 
@@ -2150,7 +2159,7 @@ mep_return_value (struct gdbarch *gdbarch, struct value *function,
 	     returned in R0.  Fetch R0's value and then read the memory
 	     at that address.  */
 	  regcache_raw_read_unsigned (regcache, MEP_R0_REGNUM, &addr);
-	  read_memory (addr, readbuf, TYPE_LENGTH (type));
+	  read_memory (addr, readbuf, type->length ());
 	}
       if (writebuf)
 	{
@@ -2222,14 +2231,14 @@ push_large_arguments (CORE_ADDR sp, int argc, struct value **argv,
 
   for (i = 0; i < argc; i++)
     {
-      unsigned arg_len = TYPE_LENGTH (value_type (argv[i]));
+      unsigned arg_len = argv[i]->type ()->length ();
 
       if (arg_len > MEP_GPR_SIZE)
 	{
 	  /* Reserve space for the copy, and then round the SP down, to
 	     make sure it's all aligned properly.  */
 	  sp = (sp - arg_len) & -4;
-	  write_memory (sp, value_contents (argv[i]), arg_len);
+	  write_memory (sp, argv[i]->contents ().data (), arg_len);
 	  copy[i] = sp;
 	}
     }
@@ -2258,7 +2267,7 @@ mep_push_dummy_call (struct gdbarch *gdbarch, struct value *function,
   /* The address of the end of the stack area for arguments.  This is
      just for error checking.  */
   CORE_ADDR arg_stack_end;
-  
+
   sp = push_large_arguments (sp, argc, argv, copy);
 
   /* Reserve space for the stack arguments, if any.  */
@@ -2282,9 +2291,9 @@ mep_push_dummy_call (struct gdbarch *gdbarch, struct value *function,
       ULONGEST value;
 
       /* Arguments that fit in a GPR get expanded to fill the GPR.  */
-      if (TYPE_LENGTH (value_type (argv[i])) <= MEP_GPR_SIZE)
-	value = extract_unsigned_integer (value_contents (argv[i]),
-					  TYPE_LENGTH (value_type (argv[i])),
+      if (argv[i]->type ()->length () <= MEP_GPR_SIZE)
+	value = extract_unsigned_integer (argv[i]->contents ().data (),
+					  argv[i]->type ()->length (),
 					  byte_order);
 
       /* Arguments too large to fit in a GPR get copied to the stack,
@@ -2314,7 +2323,7 @@ mep_push_dummy_call (struct gdbarch *gdbarch, struct value *function,
 
   /* Update the stack pointer.  */
   regcache_cooked_write_unsigned (regcache, MEP_SP_REGNUM, sp);
-  
+
   return sp;
 }
 
@@ -2325,9 +2334,6 @@ mep_push_dummy_call (struct gdbarch *gdbarch, struct value *function,
 static struct gdbarch *
 mep_gdbarch_init (struct gdbarch_info info, struct gdbarch_list *arches)
 {
-  struct gdbarch *gdbarch;
-  struct gdbarch_tdep *tdep;
-
   /* Which me_module are we building a gdbarch object for?  */
   CONFIG_ATTR me_module;
 
@@ -2362,8 +2368,8 @@ mep_gdbarch_init (struct gdbarch_info info, struct gdbarch_list *arches)
 	  const char *file_name = bfd_get_filename (info.abfd);
 	  const char *file_endianness
 	    = bfd_big_endian (info.abfd) ? "big" : "little";
-	  
-	  fputc_unfiltered ('\n', gdb_stderr);
+
+	  gdb_putc ('\n', gdb_stderr);
 	  if (module_name)
 	    warning (_("the MeP module '%s' is %s-endian, but the executable\n"
 		       "%s is %s-endian."),
@@ -2381,14 +2387,20 @@ mep_gdbarch_init (struct gdbarch_info info, struct gdbarch_list *arches)
      already.  info->bfd_arch_info needs to match, but we also want
      the right me_module: the ELF header's e_flags field needs to
      match as well.  */
-  for (arches = gdbarch_list_lookup_by_info (arches, &info); 
+  for (arches = gdbarch_list_lookup_by_info (arches, &info);
        arches != NULL;
        arches = gdbarch_list_lookup_by_info (arches->next, &info))
-    if (gdbarch_tdep (arches->gdbarch)->me_module == me_module)
-      return arches->gdbarch;
+    {
+      mep_gdbarch_tdep *tdep
+	= gdbarch_tdep<mep_gdbarch_tdep> (arches->gdbarch);
 
-  tdep = XCNEW (struct gdbarch_tdep);
-  gdbarch = gdbarch_alloc (&info, tdep);
+      if (tdep->me_module == me_module)
+	return arches->gdbarch;
+    }
+
+  gdbarch *gdbarch
+    = gdbarch_alloc (&info, gdbarch_tdep_up (new mep_gdbarch_tdep));
+  mep_gdbarch_tdep *tdep = gdbarch_tdep<mep_gdbarch_tdep> (gdbarch);
 
   /* Get a CGEN CPU descriptor for this architecture.  */
   {
@@ -2412,21 +2424,18 @@ mep_gdbarch_init (struct gdbarch_info info, struct gdbarch_list *arches)
   set_gdbarch_register_type (gdbarch, mep_register_type);
   set_gdbarch_num_pseudo_regs (gdbarch, MEP_NUM_PSEUDO_REGS);
   set_gdbarch_pseudo_register_read (gdbarch, mep_pseudo_register_read);
-  set_gdbarch_pseudo_register_write (gdbarch, mep_pseudo_register_write);
+  set_gdbarch_deprecated_pseudo_register_write (gdbarch,
+						mep_pseudo_register_write);
   set_gdbarch_dwarf2_reg_to_regnum (gdbarch, mep_debug_reg_to_regnum);
   set_gdbarch_stab_reg_to_regnum (gdbarch, mep_debug_reg_to_regnum);
 
   set_gdbarch_register_reggroup_p (gdbarch, mep_register_reggroup_p);
-  reggroup_add (gdbarch, all_reggroup);
-  reggroup_add (gdbarch, general_reggroup);
-  reggroup_add (gdbarch, save_reggroup);
-  reggroup_add (gdbarch, restore_reggroup);
   reggroup_add (gdbarch, mep_csr_reggroup);
   reggroup_add (gdbarch, mep_cr_reggroup);
   reggroup_add (gdbarch, mep_ccr_reggroup);
 
   /* Disassembly.  */
-  set_gdbarch_print_insn (gdbarch, mep_gdb_print_insn); 
+  set_gdbarch_print_insn (gdbarch, mep_gdb_print_insn);
 
   /* Breakpoints.  */
   set_gdbarch_breakpoint_kind_from_pc (gdbarch, mep_breakpoint::kind_from_pc);
@@ -2441,7 +2450,7 @@ mep_gdbarch_init (struct gdbarch_info info, struct gdbarch_list *arches)
 
   /* Return values.  */
   set_gdbarch_return_value (gdbarch, mep_return_value);
-  
+
   /* Inferior function calls.  */
   set_gdbarch_frame_align (gdbarch, mep_frame_align);
   set_gdbarch_push_dummy_call (gdbarch, mep_push_dummy_call);
@@ -2449,15 +2458,13 @@ mep_gdbarch_init (struct gdbarch_info info, struct gdbarch_list *arches)
   return gdbarch;
 }
 
-void _initialize_mep_tdep ();
-void
-_initialize_mep_tdep ()
+INIT_GDB_FILE (mep_tdep)
 {
   mep_csr_reggroup = reggroup_new ("csr", USER_REGGROUP);
-  mep_cr_reggroup  = reggroup_new ("cr", USER_REGGROUP); 
+  mep_cr_reggroup  = reggroup_new ("cr", USER_REGGROUP);
   mep_ccr_reggroup = reggroup_new ("ccr", USER_REGGROUP);
 
-  register_gdbarch_init (bfd_arch_mep, mep_gdbarch_init);
+  gdbarch_register (bfd_arch_mep, mep_gdbarch_init);
 
   mep_init_pseudoregister_maps ();
 }
