@@ -1,6 +1,6 @@
 /* This testcase is part of GDB, the GNU debugger.
 
-   Copyright 2008-2019 Free Software Foundation, Inc.
+   Copyright 2008-2026 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -40,10 +40,11 @@ void big_frame ()
   small_frame ();
 }
 
-void
+int
 main ()
 {
   small_frame ();
   medium_frame ();
   big_frame ();
+  return 0;
 }
