@@ -1,6 +1,6 @@
 /* Self tests for function_view for GDB, the GNU debugger.
 
-   Copyright (C) 2017-2024 Free Software Foundation, Inc.
+   Copyright (C) 2017-2026 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -250,9 +250,7 @@ run_tests ()
 } /* namespace function_view */
 } /* namespace selftests */
 
-void _initialize_function_view_selftests ();
-void
-_initialize_function_view_selftests ()
+INIT_GDB_FILE (function_view_selftests)
 {
   selftests::register_test ("function_view",
 			    selftests::function_view::run_tests);
