@@ -1,5 +1,5 @@
 /* I/O, string, cleanup, and other random utilities for GDB.
-   Copyright (C) 1986-2025 Free Software Foundation, Inc.
+   Copyright (C) 1986-2026 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -157,9 +157,10 @@ extern void wrap_here (int);
 
 extern void reinitialize_more_filter (void);
 
-/* Return the number of characters in a line.  */
+/* Return the number of characters in a line.  Will never be zero, but can
+   be UINT_MAX, which indicates unlimited characters per line.  */
 
-extern int get_chars_per_line ();
+extern unsigned int get_chars_per_line ();
 
 extern bool pagination_enabled;
 
@@ -407,14 +408,11 @@ class scoped_restore_warning_hook
 public:
   explicit scoped_restore_warning_hook (warning_hook_handler new_handler);
 
+  DISABLE_COPY_AND_ASSIGN (scoped_restore_warning_hook);
+
   ~scoped_restore_warning_hook ();
 
 private:
-  scoped_restore_warning_hook (const scoped_restore_warning_hook &other)
-    = delete;
-  scoped_restore_warning_hook &operator= (const scoped_restore_warning_hook &)
-    = delete;
-
   warning_hook_handler m_save;
 };
 
